/*
	(C) 1995-96 AROS - The Amiga Research OS
	$Id: setiprefs.c,v 1.21 2020/02/04 00:37:25 cyfm Exp $

	Desc: Locks the access to all public fields of the IntuitionBase
	Lang: english
*/
#include <proto/exec.h>
#include <proto/graphics.h>
#include <proto/utility.h>
#include <proto/layers.h>
#include <prefs/palette.h>
#include <intuition/pointerclass.h>
#include <devices/input.h>
#include <exec/io.h>
#include <string.h>
#include "intuition_intern.h"
#include "menus.h"
#include "inputhandler_support.h"
#include "pointers.h"
#include <private/prefs/setiprefs.h>
#ifdef __MORPHOS__
#include <cybergraphx/cybergraphics.h>
#include <ppcinline/cybergraphics.h>
#endif
#undef DEBUG
#define DEBUG 1
#include <aros/debug.h>

#if 0
struct InputPrefs
{
	char           ip_Keymap[16];
	UWORD          ip_PointerTicks;
	struct timeval ip_DoubleClick;
	struct timeval ip_KeyRptDelay;
	struct timeval ip_KeyRptSpeed;
	WORD           ip_MouseAccel;
};
#endif
#pragma pack(2)
struct IOldFontPrefs
{
	struct TextAttr fp_TextAttr;
	UBYTE           fp_Name[32];
	ULONG           fp_NotUsed;
	WORD            fp_Type;
};

struct IFontPrefs
{
	struct TextAttr fp_TextAttr;
	UBYTE           fp_Name[32];
	ULONG           fp_xxx;
	BOOL            fp_ScrFont;
};

#pragma pack()

struct IPointerColorPrefs
{
	UWORD Num;
	UWORD Red;
	UWORD Green;
	UWORD Blue;
	UWORD NotZero;
	UWORD Pad[3];
};

struct IOldPointerPrefs
{
	struct BitMap *BitMap;
	WORD  XOffset;
	WORD  YOffset;
	UWORD WordWidth;
	UWORD XResolution;
	UWORD YResolution;
	UWORD Type;
};

struct IPointerPrefs
{
	struct BitMap *BitMap;
	WORD  XOffset;
	WORD  YOffset;
	UWORD BytesPerRow;
	UWORD Size;
	UWORD YSize;
	UWORD Which;
	ULONG Zero;
};

struct IOldPenPrefs
{
	UWORD Count;
	UWORD Type;
	ULONG Pad;
	UWORD PenTable[NUMDRIPENS+1];
};

struct IOldOverScanPrefs
{
	ULONG DisplayID;
	Point ViewPos;
	Point Text;
	struct Rectangle Standard;
};

/*****i***********************************************************************

	NAME */
#include <proto/intuition.h>

AROS_LH3(ULONG, SetIPrefs,

		 /*  SYNOPSIS */
		 AROS_LHA(APTR , data, A0),
		 AROS_LHA(ULONG, length, D0),
		 AROS_LHA(ULONG, type, D1),

		 /*  LOCATION */
		 struct IntuitionBase *, IntuitionBase, 96, Intuition)

/*  FUNCTION

	INPUTS

	RESULT
	Depending on the operation

	NOTES

	EXAMPLE

	BUGS

	SEE ALSO

	INTERNALS

	HISTORY
	29-10-95    digulla automatically created from
			    intuition_lib.fd and clib/intuition_protos.h

*****************************************************************************/
{
	AROS_LIBFUNC_INIT
	AROS_LIBBASE_EXT_DECL(struct IntuitionBase *,IntuitionBase)
	ULONG Result=TRUE;
	ULONG lock = LockIBase(0);

	DEBUG_SETIPREFS(bug("SetIPrefs: data %p length %lu type %lu\n", data, length, type));

	switch (type)
	{
	case IP_SCREENMODE:
		{
			DEBUG_SETIPREFS(bug("SetIPrefs: IP_SCREENMODE\n"));
			if (length >= sizeof(struct IScreenModePrefs))
			{
				struct IScreenModePrefs *mp = (struct IScreenModePrefs *)data;
				LOCKWINDOW;
				// clear some stuff, yuk
				IBase->AmbientScreenMode.ModeMonitor[0] = 0;
				IBase->AmbientScreenMode.ModeWidth = 0;
				IBase->AmbientScreenMode.ModeHeight = 0;
				IBase->AmbientScreenMode.ModeDepth = 32;
				IBase->AmbientScreenMode.OverscanType = 0;
				IBase->AmbientScreenMode.Compositing = TRUE;
				IBase->AmbientScreenMode.DisappearingTitlebar = FALSE;
				
				// translate/copy entries
				IBase->AmbientScreenMode.ModeID = mp->smp_DisplayID;
				IBase->AmbientScreenMode.DisplayWidth = mp->smp_Width;
				IBase->AmbientScreenMode.DisplayHeight = mp->smp_Height;
				IBase->AmbientScreenMode.DisplayDepth = mp->smp_Depth;
				IBase->AmbientScreenMode.AutoScroll = mp->smp_Control & AUTOSCROLL;
				UNLOCKWINDOW;
			}
		}
		break;

#if 0
	case IP_OLDSCREENMODE:
		if (length > sizeof (struct IScreenModePrefs))
			length = sizeof (struct IScreenModePrefs);
		CopyMem(&IBase->ScreenModePrefs,data,length);
		break;
#endif

	case IP_NEWSCREENTAGS:
		{
			struct TagItem *tags = (struct TagItem *)data;
			struct TagItem *tag;
			BOOL changed = FALSE;

			LOCKWINDOW;

			while ((tag = NextTagItem((struct TagItem **)&tags)))
			{
				switch (tag->ti_Tag)
				{
					case IPA_Skin:
						if (tag->ti_Data)
						{
							if (*(char *)tag->ti_Data && strcmp((char *)tag->ti_Data,IBase->AmbientScreenMode.Skin))
							{	
								//dprintf("%s/%d: %s vs %s\n",__func__, __LINE__, (char *)tag->ti_Data, IBase->AmbientScreenMode.Skin);
								changed = TRUE;
								stccpy(IBase->AmbientScreenMode.Skin,(char*)tag->ti_Data,PSD_MAXLEN_SKIN);
							}
							else if (*(char *)tag->ti_Data == 0 && strcmp("Origo",IBase->AmbientScreenMode.Skin))
							{
								stccpy(IBase->AmbientScreenMode.Skin,"Origo",PSD_MAXLEN_SKIN);
								changed = TRUE;
								//dprintf("reset skin\n");
							}
						}
						// erasing not supported!
						break;

					case IPA_ModeMonitor:
						if (tag->ti_Data)
						{
							if (strcmp((char *)tag->ti_Data,IBase->AmbientScreenMode.ModeMonitor)) 
							{ 
								//dprintf("%s/%d: %s vs %s\n",__func__, __LINE__, (char *)tag->ti_Data, IBase->AmbientScreenMode.ModeMonitor);
								changed = TRUE; 
								stccpy(IBase->AmbientScreenMode.ModeMonitor,(char*)tag->ti_Data,PSD_MAXLEN_MONITOR);
							}
						}
						else
						{
							if (IBase->AmbientScreenMode.ModeMonitor[0]) 
							{
								//dprintf("%s/%d: was %s\n",__func__, __LINE__, IBase->AmbientScreenMode.ModeMonitor);
								changed = TRUE;
							}
							IBase->AmbientScreenMode.ModeMonitor[0] = 0;
						}
						break;

#define IPA_COPY(__arg) case IPA_##__arg: if (IBase->AmbientScreenMode.__arg != tag->ti_Data) { changed = TRUE; IBase->AmbientScreenMode.__arg = tag->ti_Data; } break
// #define IPA_COPY(__arg) case IPA_##__arg: if (IBase->AmbientScreenMode.__arg != tag->ti_Data) { changed = TRUE; dprintf("%s/%d: %ld v %ld\n", __func__,__LINE__, (ULONG)IBase->AmbientScreenMode.__arg, tag->ti_Data); IBase->AmbientScreenMode.__arg = tag->ti_Data; } break
						IPA_COPY(ModeWidth);
						IPA_COPY(ModeHeight);
						IPA_COPY(ModeDepth);
						IPA_COPY(DisplayWidth);
						IPA_COPY(DisplayHeight);
						IPA_COPY(DisplayDepth);
						IPA_COPY(ModeID);
						IPA_COPY(OverscanType);
						IPA_COPY(AutoScroll);
						IPA_COPY(Compositing);
						IPA_COPY(DisappearingTitlebar);
				}
			}

			UNLOCKWINDOW;

			Result = changed ? TRUE : FALSE;
		}
		break;

	case IP_OLDICONTROL:
		DEBUG_SETIPREFS(bug("SetIPrefs: IP_OLDICONTROL\n"));
		if (length > sizeof(struct IIControlPrefs))
			length = sizeof(struct IIControlPrefs);
		CopyMem(&IBase->IControlPrefs, data, length);
		break;

	case IP_ICONTROL:
		DEBUG_SETIPREFS(bug("SetIPrefs: IP_ICONTROL\n"));
		if (length > sizeof(struct IIControlPrefs))
			length = sizeof(struct IIControlPrefs);
		CopyMem(data, &IBase->IControlPrefs, length);
		break;

#ifdef SKINS
	case IP_IEXTENSIONS:
		DEBUG_SETIPREFS(bug("SetIPrefs: IP_IEXTENSIONS\n"));
		if (length > sizeof(struct IControlExtensions))
			length = sizeof(struct IControlExtensions);
		CopyMem(data, &IBase->IControlExtensions, length);

		break;

	case IP_EMERGENCYBOOT:
		if (length == sizeof (struct EmergencyBoot))
		{
			CopyMem(data,&IBase->EmergencyBoot,length);
		}
		break;

	case IP_INPUTEXT:
		DEBUG_SETIPREFS(bug("SetIPrefs: IP_INPUTEXT\n"));
		if (length > sizeof(struct InputPrefsExt))
			length = sizeof(struct InputPrefsExt);

		CopyMem(data, &IBase->InputPrefsExt, length);

		{
			struct IOStdReq req;

			memclr(&req,sizeof (struct IOStdReq));

			req.io_Device = IBase->InputIO->io_Device;
			req.io_Unit = IBase->InputIO->io_Unit;
			req.io_Command = IND_SETMOUSETYPE;
			req.io_Data = &IBase->InputPrefsExt.ip_MouseMode;
			DoIO((struct IORequest *)&req);
		}
		break;

	case IP_INPUTMOUSE:
		DEBUG_SETIPREFS(bug("SetIPrefs: IP_INPUTMOUSE\n"));
		if (length > sizeof(struct InputMousePrefs))
			length = sizeof(struct InputMousePrefs);

		CopyMem(data, &IBase->InputMousePrefs, length);

		{
			struct IOStdReq req;

			memclr(&req,sizeof (struct IOStdReq));

			req.io_Device = IBase->InputIO->io_Device;
			req.io_Unit = IBase->InputIO->io_Unit;
			req.io_Command = IND_SETMOUSETYPE;
			req.io_Data = &IBase->InputMousePrefs.ip_MouseMode;
			DoIO((struct IORequest *)&req);

			IBase->ActivePreferences.DoubleClick = IBase->InputMousePrefs.ip_DoubleClick;

			if (!(IBase->InputMousePrefs.ip_Flags & IPMF_MOUSESCALING))
			{
				//convert old mode to new mode (mousescaling)
				WORD newticks = 2000;

				switch (IBase->InputMousePrefs.ip_PointerTicks)
				{
					case 1:
						newticks = (IBase->InputMousePrefs.ip_MouseAccel) ? 3500 : 3000;
						break;

					case 2:
						newticks = (IBase->InputMousePrefs.ip_MouseAccel) ? 2000 : 1000;
						break;

					default:
						newticks = (IBase->InputMousePrefs.ip_MouseAccel) ? 1000 : 500;
						break;
				}

				IBase->InputMousePrefs.ip_PointerTicks = newticks;
				IBase->InputMousePrefs.ip_Flags |= IPMF_MOUSESCALING;
			}

			#if 0
			/* jDc: these aren't used anymore */
			IBase->ActivePreferences.PointerTicks = IBase->InputMousePrefs.ip_PointerTicks;

			if (IBase->InputMousePrefs.ip_MouseAccel)
			{
				IBase->ActivePreferences.EnableCLI |= MOUSE_ACCEL;
			} else {
				IBase->ActivePreferences.EnableCLI &= ~MOUSE_ACCEL;
			}
			#endif

			int_SkinAction(SKA_ReloadPointers,NULL,NULL,IntuitionBase);
		}
		break;

	case IP_MOUSESPEED:
		if (length >= sizeof (struct InputMousePrefs))
		{
			IBase->InputMousePrefs.ip_PointerTicks = ((struct InputMousePrefs *)data)->ip_PointerTicks;
		}
		break;

	case IP_OLDMOUSESPEED:
		if (length >= sizeof (struct InputMousePrefs))
		{
			((struct InputMousePrefs *)data)->ip_PointerTicks = IBase->InputMousePrefs.ip_PointerTicks;
		}
		break;

	case IP_INPUTKEYBOARD:
		DEBUG_SETIPREFS(bug("SetIPrefs: IP_INPUTKEYBOARD\n"));
		if (length > sizeof(struct InputKeyboardPrefs))
			length = sizeof(struct InputKeyboardPrefs);

		CopyMem(data, &IBase->InputKeyboardPrefs, length);

		{
			struct Preferences prefs;
			GetPrefs(&prefs,sizeof (prefs));
			prefs.KeyRptDelay = IBase->InputKeyboardPrefs.ip_KeyRptDelay;
			prefs.KeyRptSpeed = IBase->InputKeyboardPrefs.ip_KeyRptSpeed;
			SetPrefs(&prefs,sizeof (prefs),FALSE);
		}

		break;
#if 0
	case IP_OLDSKINNAME:
		{
			struct IntScreen *wbs;

			DEBUG_SETIPREFS(bug("SetIPrefs: IP_OLDSKINNAME\n"));
			LOCKWINDOW;
			wbs = IS(IBase->WorkBench);

			if (wbs)
			{
				stccpy((char*)data,wbs->DInfo.dri_Customize->skinname,length);
			} else {
				stccpy((char*)data,IBase->AmbientScreenMode.Skin,length);
			}

			UNLOCKWINDOW;
		}
		break;

	case IP_SKINNAME:
		DEBUG_SETIPREFS(bug("SetIPrefs: IP_SKINNAME\n"));

		if (length > IC_SKINNAMELEN) length = IC_SKINNAMELEN;

		LOCKWINDOW;
		stccpy(IBase->AmbientScreenMode.Skin,(char*)data,length);
		IBase->AmbientScreenMode.Skin[length-1] = 0;
		UNLOCKWINDOW;

		break;
#endif
	case IP_IACTIONS:
		DEBUG_SETIPREFS(bug("SetIPrefs: IP_IACTIONS\n"));
		{
			ULONG mem;

			ObtainSemaphore(&IBase->InputHandlerLock);

			mem = IBase->NumIControlActions * (sizeof (struct IAction));

			IBase->NumIControlActions = 0;

			if (IBase->IControlActions)
			{
				FreeMem(IBase->IControlActions,mem);
			}

			mem = (length/(sizeof (struct IAction)) * (sizeof (struct IAction)));

			if ((IBase->IControlActions = AllocMem(mem,MEMF_ANY)))
			{

				IBase->NumIControlActions = mem / (sizeof (struct IAction));
				CopyMem(data, IBase->IControlActions, mem);
			}

			ReleaseSemaphore(&IBase->InputHandlerLock);
		}
		break;
#endif /* SKINS */

	case IP_FONT:
		DEBUG_SETIPREFS(bug("SetIPrefs: IP_FONT\n"));
		{
			struct IOldFontPrefs *fp = data;
			struct TextFont *font = OpenFont(&fp->fp_TextAttr);

			DEBUG_SETIPREFS(bug("SetIPrefs: Type %d Name <%s> Size %d Font %p\n", fp->fp_Type, fp->fp_Name, fp->fp_TextAttr.ta_YSize, font));

			if (font)
			{
				struct TextFont **fontptr;
				struct TextFont *oldfont;

				Forbid();

				if (fp->fp_Type==0)
				{
					/*
					 * We can't free graphics defaultfont..it's shared
					 */
					fontptr = &GfxBase->DefaultFont;
					oldfont = NULL;
				}
				else
				{
					fontptr = &IBase->ScreenFont;
					oldfont = *fontptr;
				}

				*fontptr = font;

				Permit();

				if (oldfont)
				{
					CloseFont(oldfont);
				}
			}
		}
		break;

	#if 0
	case IP_FONT:
		DEBUG_SETIPREFS(bug("SetIPrefs: IP_FONT\n"));
		{
			struct IFontPrefs *fp = data;
			struct TextFont *font = OpenFont(&fp->fp_TextAttr);
			struct TextFont **fontptr;

			DEBUG_SETIPREFS(bug("SetIPrefs: Type %d Name <%s> Size %d Font %p\n", fp->fp_ScrFont, fp->fp_Name, fp->fp_TextAttr.ta_YSize, font));

			if (font)
			{
				if (fp->fp_ScrFont==0)
				{
					/*
					 * We can't free graphics defaultfont..it's shared
					 */
					fontptr = &GfxBase->DefaultFont;
				}
				else
				{
					fontptr = &IBase->ScreenFont;
					CloseFont(*fontptr);
				}
				*fontptr = font;
			}
		}
		break;
		#endif

	case IP_POINTER:
		DEBUG_SETIPREFS(bug("SetIPrefs: IP_OLDPOINTER\n"));
		{
#if 0
			struct IOldPointerPrefs *fp = data;
			Object *pointer;
			#ifndef __MORPHOS__
			Object **oldptr;
			#endif

			DEBUG_SETIPREFS(bug("SetIPrefs: Bitmap 0x%lx XOffset %ld YOffset %ld XResolution %ld YResolution %ld WordWidth %ld Type %ld\n",
					    (ULONG) fp->BitMap,
					    (LONG) fp->XOffset,
					    (LONG) fp->YOffset,
					    (LONG) fp->XResolution,
					    (LONG) fp->YResolution,
					    (LONG) fp->WordWidth,
					    (LONG) fp->Type));

			{
				struct TagItem pointertags[] =
				    {
					    {POINTERA_BitMap , (ULONG) fp->BitMap},
					    {POINTERA_XOffset		, fp->XOffset	    },
					    {POINTERA_YOffset		, fp->YOffset	    },
					    {POINTERA_XResolution	, fp->XResolution   },
					    {POINTERA_YResolution	, fp->YResolution   },
					    {POINTERA_WordWidth		, fp->WordWidth     },
					    {TAG_DONE					    }
				    };

				pointer = NewObjectA(
					      IBase->pointerclass,
					      NULL,
					      pointertags);

			}

#ifdef SKINS
				InstallPointerMOS(IntuitionBase, fp->Type ? POINTERTYPE_BUSY : POINTERTYPE_NORMAL, pointer);
#else
			oldptr = fp->Type ?
				 &IBase->BusyPointer :
				 &IBase->DefaultPointer;
				InstallPointer(IntuitionBase, oldptr, pointer);
#endif
#endif

			/*
			 * Original iprefs checks for a 0 or -1 return
			 * otherwise it expects a returned bitmap..sigh
			 */
			Result = -1;
		}
		break;

	#if 0
	case IP_POINTER:
		DEBUG_SETIPREFS(bug("SetIPrefs: IP_POINTER\n"));
		{
			struct IPointerPrefs *fp = data;
			struct TagItem pointertags[] =
			    {
				    {
					    POINTERA_BitMap    , (ULONG) fp->BitMap
				    },
				    {POINTERA_XOffset   , fp->XOffset   },
				    {POINTERA_YOffset   , fp->YOffset   },
				    {TAG_DONE				}
			    };

			Object *pointer = NewObjectA(
					      IBase->pointerclass,
					      NULL,
					      pointertags);

#ifdef SKINS
			Object ** oldptr = fp->Which ?
				 &IBase->Pointers[POINTERTYPE_BUSY] :
				 &IBase->Pointers[POINTERTYPE_NORMAL];
#else
			Object ** oldptr = fp->Which ?
				 &IBase->BusyPointer :
				 &IBase->DefaultPointer;
#endif

			InstallPointer(IntuitionBase, oldptr, pointer);
		}
		break;
	#endif

	case IP_PENS:
		DEBUG_SETIPREFS(bug("SetIPrefs: IP_OLDPENS\n"));
		{
			struct IOldPenPrefs *fp = data;
			UWORD *dataptr;
			int i;
			DEBUG_SETIPREFS(bug("SetIPrefs: Count %ld Type %ld\n",
					    (LONG) fp->Count,
					    (LONG) fp->Type));

			if (fp->Type==0)
			{
				dataptr = &IBase->DriPens4[0];
				DEBUG_SETIPREFS(bug("SetIPrefs: Pens4[]\n"));
			}
			else
			{
				dataptr = &IBase->DriPens8[0];
				DEBUG_SETIPREFS(bug("SetIPrefs: Pens8[]\n"));
			}
			for (i=0;i<NUMDRIPENS;i++)
			{
				if (fp->PenTable[i]==(UWORD)~0UL)
				{
					/*
					 * end of the array
					 */
					DEBUG_SETIPREFS(bug("SetIPrefs: PenTable end at entry %ld\n", (LONG) i));
					break;
				}
				else
				{
					DEBUG_SETIPREFS(bug("SetIPrefs: Pens[%ld] %ld\n",
							    (LONG) i,
							    (LONG) fp->PenTable[i]));
					dataptr[i] = fp->PenTable[i];
				}
			}
		}
		break;

	case IP_OVERSCAN:
		DEBUG_SETIPREFS(bug("SetIPrefs: IP_OLDOVERSCAN\n"));
		{
			struct IOldOverScanPrefs *fp = data;
			DEBUG_SETIPREFS(bug("SetIPrefs: DisplayID 0x%lx\n",
					    fp->DisplayID));

			DEBUG_SETIPREFS(bug("SetIPrefs: ViewPos.x %ld ViewPos.y %ld\n",
					    (LONG) fp->ViewPos.x,
					    (LONG) fp->ViewPos.y));

			DEBUG_SETIPREFS(bug("SetIPrefs: Text.x %ld Text.y %ld\n",
					    (LONG) fp->Text.x,
					    (LONG) fp->Text.y));

			DEBUG_SETIPREFS(bug("SetIPrefs: MinX %ld MinY %ld MaxX %ld MaxY %ld\n",
					    (LONG) fp->Standard.MinX,
					    (LONG) fp->Standard.MinY,
					    (LONG) fp->Standard.MaxX,
					    (LONG) fp->Standard.MaxY));
		}
		break;

	case IP_PTRCOLOR:
		DEBUG_SETIPREFS(bug("SetIPrefs: IP_PTRCOLOR\n"));
		if (IntuitionBase->ActiveScreen)
		{
			struct IPointerColorPrefs *fp = data;
			if (fp->Num >= 8 && fp->Num <= 10)
			{
				(&IBase->ActivePreferences.color17)[fp->Num - 8] =
				    ((fp->Red << 4) & 0xf00) | (fp->Green & 0x0f0) | (fp->Blue >> 4);
				SetPointerColors(IntuitionBase);
			}
		}
		break;

	case IP_PALETTE:
		DEBUG_SETIPREFS(bug("SetIPrefs: IP_OLDPALETTE\n"));
		{
			struct ColorSpec *pp = data;
			struct Color32 *p = IBase->Colors;

			// COLORTABLEENTRIES == 32

			DEBUG_SETIPREFS(bug("SetIPrefs: Intuition Color32 Table 0x%lx\n", (ULONG) p));

			while (pp->ColorIndex != ~0x0)
			{
				DEBUG_SETIPREFS(bug("SetIPrefs: Index %ld Red 0x%lx Green 0x%lx Blue 0x%lx\n",
						    (LONG) pp->ColorIndex,
						    (ULONG) pp->Red,
						    (ULONG) pp->Green,
						    (ULONG) pp->Blue));
				if (pp->ColorIndex < COLORTABLEENTRIES)
				{
					struct Preferences *ActivePrefs;

					p[pp->ColorIndex].red   = (pp->Red<<16)|pp->Red;
					p[pp->ColorIndex].green = (pp->Green<<16)|pp->Green;
					p[pp->ColorIndex].blue  = (pp->Blue<<16)|pp->Blue;

					// check for pointer colors
					if ((ActivePrefs = &IBase->ActivePreferences))
					{
						if (pp->ColorIndex >= 8 && pp->ColorIndex <= 10)
						{
							UWORD *ptrcols=&ActivePrefs->color17;

							ptrcols[pp->ColorIndex - 8] =
							    ((pp->Red >> 4) & 0xf00) | ((pp->Green >> 8) & 0x0f0) | (pp->Blue >> 12);


							if (IntuitionBase->ActiveScreen)
							{
								SetPointerColors(IntuitionBase);
							}
						}
					}
					DEBUG_SETIPREFS(bug("SetIPrefs: Set Color32 %ld Red 0x%lx Green 0x%lx Blue 0x%lx\n",
							    (LONG) pp->ColorIndex,
							    p[pp->ColorIndex].red,
							    p[pp->ColorIndex].green,
							    p[pp->ColorIndex].blue));
				}
				else
				{
					DEBUG_SETIPREFS(bug("SetIPrefs: ColorIndex %ld > TableSize %ld\n",
					                    (LONG) pp->ColorIndex, (LONG) COLORTABLEENTRIES));
				}
				pp++;
			}
		}
		break;

	#if 0
	case IP_PALETTE:
		DEBUG_SETIPREFS(bug("SetIPrefs: IP_PALETTE\n"));
		{
			struct PalettePrefs *pp = data;
			int k;
			struct Color32 *p = IBase->Colors;

			CopyMem(pp->pap_4ColorPens, IBase->DriPens4, NUMDRIPENS * sizeof(UWORD));
			CopyMem(pp->pap_8ColorPens, IBase->DriPens8, NUMDRIPENS * sizeof(UWORD));
			for (k = 0; k < COLORTABLEENTRIES; ++k)
			{
				int n = pp->pap_Colors[k].ColorIndex;
				if (n == -1)
					break;
				else if (n >= 0 && n < 8)
				{
					p[n].red   = pp->pap_Colors[k].Red * 0x10001;
					p[n].green = pp->pap_Colors[k].Green * 0x10001;
					p[n].blue  = pp->pap_Colors[k].Blue * 0x10001;
				}
				DEBUG_SETIPREFS(bug("SetIPrefs: Set Color32 %ld Red 0x%lx Green 0x%lx Blue 0x%lx\n",
						    (LONG) n,
						    p[n].red,
						    p[n].green,
						    p[n].blue));
			}
		}
		break;
	#endif

	case IP_OLDSERIAL:
		{
			if (length > sizeof(struct ISerialPrefs))
				length = sizeof(struct ISerialPrefs);
			CopyMem(&IBase->SerialPrefs, data, length);

		}
		break;

	case IP_SERIAL:
		DEBUG_SETIPREFS(bug("SetIPrefs: IP_SERIAL\n"));
		{
			struct ISerialPrefs *pp = data;
			ULONG	SBufSize;

			if (length > sizeof(struct ISerialPrefs))
				length = sizeof(struct ISerialPrefs);
			CopyMem(data, &IBase->SerialPrefs, length);

			/*
			 * legacy crap
			 */
			if (pp->sp_OutputBuffer <= 512)
			{
				SBufSize = SBUF_512;
			}
			else
			if (pp->sp_OutputBuffer <= 1024)
			{
				SBufSize = SBUF_1024;
			}
			else
			if (pp->sp_OutputBuffer <= 2048)
			{
				SBufSize = SBUF_2048;
			}
			else
			if (pp->sp_OutputBuffer <= 4096)
			{
				SBufSize = SBUF_4096;
			}
			else
			if (pp->sp_OutputBuffer <= 8000)
			{
				SBufSize = SBUF_8000;
			}
			else
			{
				SBufSize = SBUF_16000;
			}

			#warning "Not really sure about the Unit0Map thing"
			IBase->ActivePreferences.DefaultSerUnit = pp->sp_Unit0Map;

			if (pp->sp_BaudRate < sizeof(UWORD))
			{
				IBase->ActivePreferences.BaudRate = pp->sp_BaudRate;
			}
			else
			{
				IBase->ActivePreferences.BaudRate = 0;
			}

			IBase->ActivePreferences.SerRWBits  = (((8 - min(8,pp->sp_BitsPerChar)) << 4) & SREAD_BITS) | (((8 - min(8,pp->sp_BitsPerChar)) << 0) & SWRITE_BITS);
			IBase->ActivePreferences.SerStopBuf = ((pp->sp_StopBits << 4) & SSTOP_BITS) | (SBufSize & SBUFSIZE_BITS);
			IBase->ActivePreferences.SerParShk  = ((pp->sp_Parity << 4) & SPARITY_BITS) | ((pp->sp_InputHandshake << 0) & SHSHAKE_BITS);
		}
		break;

	case IP_CODEPAGE:
		{
			char *newcp = (char*)data;

			if (newcp && *newcp)
			{
				// note: currently not accessed by anything... therefore no protection around it
				stccpy(IBase->Codepage,newcp,sizeof (IBase->Codepage));
			}
		}
		break;

	case IP_COMPOSITINGTAGS:
		{
			struct TagItem *tags = (struct TagItem *)data;
			struct TagItem *tag;
			BOOL changed = FALSE;

			LOCKWINDOW;

			while ((tag = NextTagItem((struct TagItem **)&tags)))
			{
				#define IPAC(__arg,__destarg) case IPAC_##__arg: if (IBase->LayerSettings.__destarg != tag->ti_Data) changed = TRUE; IBase->LayerSettings.__destarg = tag->ti_Data; break
				switch (tag->ti_Tag)
				{
					IPAC(UseDoubleBugger,AllowDoubleBuffered);
					IPAC(UseTripleBugger,AllowTripleBuffered);
					IPAC(IgnoreMemoryChecks,IgnoreMemoryChecks);
					IPAC(LoadBalancing,LoadBalancing);
				}
			}

			if (changed)
			{
				struct Screen *scr;
				struct TagItem settings[] =
				{
					{LT_AllowDoubleBuffered, IBase->LayerSettings.AllowDoubleBuffered},
					{LT_AllowTripleBuffered, IBase->LayerSettings.AllowTripleBuffered},
					{LT_IgnoreMemoryChecks, IBase->LayerSettings.IgnoreMemoryChecks},
					{LT_LoadBalancing, IBase->LayerSettings.LoadBalancing},
					{TAG_DONE},
				};

				for (scr = IntuitionBase->FirstScreen; scr != NULL; scr = scr->NextScreen)
				{
					LayersControlTagList(&scr->LayerInfo,settings);
				}
			}

			UNLOCKWINDOW;
		}
		break;

	default:
		DEBUG_SETIPREFS(bug("SetIPrefs: Unknown Prefs Type\n"));
		Result = FALSE;
		break;
	}

	UnlockIBase(lock);

	switch (type)
	{
	case IP_BLANKER:
		// done outside of ibase lock since it needs the ih lock!
		if (length == sizeof (struct BlankerPrefs))
		{
			LOCKIH;
			CopyMem(data,&IBase->BlankerPrefs,length);
			InitBlankerPrefs(IntuitionBase);
			QueryBlankerPrefs(IntuitionBase);
			UNLOCKIH;
		}
		break;

	}

	DEBUG_SETIPREFS(bug("SetIPrefs: Result 0x%lx\n",Result));
	return(Result);
	AROS_LIBFUNC_EXIT
} /* private1 */

